/*
 * Process Variable Library with JVisu Socket Server
 *
 * Copyright (C) 2003 Thomas Gleixner <tglx@linutronix.de>
 *           (C) 2005 Robert Schwebel <r.schwebel@pengutronix.de>
 *
 * file: pv_bs_std.c
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 *
 * PURPOSE:
 * Handling files on a standard filesystem media (backing store).
 *
 */

#include <libpv_config.h>

#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>

#include <pv_bs_std.h>


struct pv_bs_std_priv {
	int fd;
	int mmap;
};


static int pv_bs_std_exit(struct pv_backing_store	*store)
{
	struct pv_bs_std_priv	*priv = store->priv;

	if(priv->mmap)
		munmap(store->map, store->size);

	close(priv->fd);
	free(priv);

	return 0;
}


static int pv_bs_std_read(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_bs_std_priv	*priv = store->priv;
	int			err;

	dprintf(LEVEL_ALL, "(%s) off %d, len %zd\n",
		__FUNCTION__, (int)offset, len);

	err = lseek(priv->fd, offset, SEEK_SET);
	if(err < 0) {
		perror("lseek");
		return err;
	}

	err = read(priv->fd, store->map + offset, len);
	if(err < 0) {
		perror("write");
		return err;
	}

	return 0;
}


static int pv_bs_std_write(pv_backing_store_t *store, off_t offset, size_t len) {
	struct pv_bs_std_priv	*priv = store->priv;
	int			err;

	dprintf(LEVEL_ALL, "(%s) off %d, len %zd\n",
		__FUNCTION__, (int)offset, len);

	err = lseek(priv->fd, offset, SEEK_SET);
	if(err < 0) {
		perror("lseek");
		return err;
	}

	err = write(priv->fd,store->map + offset, len);
	if(err < 0) {
		perror("write");
		return err;
	}

	return 0;
}

/**
 * pv_bs_std_init - open process image on standard rw backing store
 * @store:
 * @filename:
 * @size:
 *
 * Description: Opens the given file and tries to mmap
 * it into process space. mmap could fail in the case
 * the filesystem doesn't support this feature.
 *
 * Return value:
 **/
int pv_bs_std_init(struct pv_backing_store	*store,
		   const char			*filename,
		   size_t			size)
{
	struct pv_bs_std_priv	*priv;
	int			err;

	dprintf(LEVEL_INFO, "(%s) initialising standard backing store\n",
		__FUNCTION__);

	priv = calloc(1, sizeof(struct pv_bs_std_priv));
	if(!priv) {
		err = ENOMEM;
		dprintf(LEVEL_CRIT, "(%s) failed to alloc private mem, err %d\n",
			__FUNCTION__, err);
		goto exit;
	}

	store->priv  = priv;
	store->exit  = pv_bs_std_exit;
	store->read  = pv_bs_std_read;
	store->write = pv_bs_std_write;
	store->size  = size;
   /*
	* Open the given file and try to map
	* it into our process space
	*/
	err = open(filename, O_RDWR | O_SYNC);
	if(err < 0) {
		err = -errno;
		dprintf(LEVEL_CRIT, "(%s) failed to open backing store \"%s\", err %d\n",
			__FUNCTION__, filename, err);
		goto exit_free;
	}
	priv->fd = err;
   /*
    * we need a mapped file with read and(!) write access
	* This fails on a jffs2 filesystem for example
	*/
	store->map = mmap(NULL, store->size, PROT_READ | PROT_WRITE,
			  MAP_SHARED, priv->fd, 0);
	if(store->map == MAP_FAILED) {
		dprintf(LEVEL_INFO, "(%s) could not mmap, need shared memory segment\n",
			__FUNCTION__);
		store->map = NULL;	/* this is not(!) a failure */
	} else
		priv->mmap = 1;	/* mapping was successfull */

	return 0;

 exit_free:
	free(priv);
 exit:
	return err;
}

/* end of file pc_bs_std.c */
